#   htmlcc - Statically compiled HTML templates for C
#   Copyright (C) 2025  Lee Yingtong Li
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from . import Emitter, register_emitter

class CgitEmitter(Emitter):
	"""Emitter for the cgit CGI engine"""
	
	def emit_preamble(self) -> None:
		self.emit('''#define USE_THE_REPOSITORY_VARIABLE
#include "../cgit.h"
#include "../html.h"
#include "../ui-shared.h"
#include "version.h"''')
	
	def output_literal_string(self, literal_string: str) -> None:
		self.emit(f'html({self.escape_cstr(literal_string)});')
	
	def output_variable_as_attr(self, variable: str) -> None:
		self.emit(f'html_attr({variable});')
	
	def output_variable_as_text(self, variable: str) -> None:
		self.emit(f'html_txt({variable});')
	
	def output_variable_formatted(self, format_string: str, variable: str) -> None:
		self.emit(f'htmlf({self.escape_cstr(format_string)}, {variable});')
	
	def output_variable_urlencoded(self, variable: str) -> None:
		self.emit(f'html_url_arg({variable});')
	
	def start_page(self, page_name: str) -> None:
		super().start_page(page_name)
		self.emit('cgit_print_http_headers();')

register_emitter('cgit', CgitEmitter)
